#!/bin/sh
#    security_get.sh
# check for available updates and get/apply them
#
# 06/05/2017 acress - add SecurityAutoUpdate feature
# 06/07/2017 acress - clean up leftover files in /var/cache
# 06/27/2017 acress - do not clean up leftover files if bg mode
BPDIR=/usr/bp
dbg=0
vardir=/var/cache
log=/var/log/unitrends-security.log

is_activity()
{
  stats=`psql bpdb postgres -tAc "select status from bp.jobs where percent_done <> 100;"`
  nactive=0
  for s in $stats
  do
     show_status $s |head -n2 |grep -q ST_CANCELLED
     if [ $? -ne 0 ]; then
         nactive=`expr $nactive + 1`
     fi
  done
  if [ $nactive -gt 0 ]; then
     echo "There are $nactive active jobs, please wait or cancel the jobs."
     return 1
  fi

  # check if samba is in use (FLR, RDR, WinIR)
  flr=`$BPDIR/bin/cmc_vdisk_mount status |wc -l`
  if [ $flr -gt 0 ]; then
     echo "There are $flr active FLR instances, please wait or stop them."
     return 1
  fi
  $BPDIR/bin/vmware_instant_recovery status |grep -q none
  if [ $? -ne 0 ]; then
     echo "There are active VMIR instances, please wait or stop them."
     return 1
  fi
  $BPDIR/bin/hyperv_instant_recovery status |grep -q none
  if [ $? -ne 0 ]; then
     echo "There are active HVIR instances, please wait or stop them."
     return 1
  fi
  nirs=`psql bpdb postgres -tAc "select * from bp.virtual_clients where vmware_host is not NULL or hyperv_host is not NULL;" |wc -l`
  if [ $nirs -gt 0 ]; then
     echo "There are $nirs remote IR sessions, please wait or stop them."
     return 1
  fi
  #Should also check if any RDR sessions active

  return 0
}

update_available()
{
  tmpf=/tmp/wget.tmp

  cd $vardir
  rm -f security_version.txt* 
  rver0=`rpm -q unitrends-security`
  wget ftp://ftp.unitrends.com/utilities/security_version.txt >$tmpf 2>&1
  rver1=`cat security_version.txt`
  if [ $dbg -eq 1 ]; then
     echo "rver0=$rver0"
     echo "rver1=$rver1"
  fi
  cd $mydir
  if [ "$rver1" = "" ]; then
     tail -n5 $tmpf
     echo "Cannot get remote update version"
     rm -f $tmpf
     return 0
  fi
  rm -f $tmpf
  # need to tell if new version is greater or not (not equal -> greater)
  if [ "$rver0" = "$rver1" ]; then
     echo "Already have the latest update: $rver0"
     return 0
  fi
  echo "$rver1 is available" 
  return 1
}

checkAvailableUpdate() 
{
  tmpa=/tmp/security_avail.tmp
  update_available >$tmpa
  rv=$?
  msg=`cat $tmpa`
  rm -f $tmpa
  if [ $rv -eq 0 ]; then
     # already latest
     echo "$msg"
     return 0
  fi
  update=`$BPDIR/bin/bputil -g "Configuration Options" SecurityAutoUpdate 0 $BPDIR/bpinit/master.ini`
  alert=`$BPDIR/bin/bputil -g "Configuration Options" SecurityAlert 0 $BPDIR/bpinit/master.ini`
  if [ $dbg -eq 1 ]; then
	echo "alert=$alert update=$update"
  fi
  echo "$msg"
  if [ $update -gt 0 ]; then
	  applySecurityUpdate fg
  elif [ $alert -gt 0 ]; then
      # if SecurityAlert=1, send notification
      action="Go to Configure/Edit Appliance/Advanced/Support Toolbox/Security to update"
      $BPDIR/bin/send_notification.php 138 "Security Update" "$msg. $action"  >$tmpa 2>&1
      echo "Send alert (ret=$?): $msg" |tee -a $log
      rm -f $tmpa
  fi
  return 1
}

applySecurityUpdate() 
{
  tmpf=/tmp/wget.tmp
  sum0=/tmp/mdsum0.tmp
  sum1=/tmp/mdsum1.tmp
  tarball=security_updates.tar.gz
  mydir=`pwd`
  param=$1

  update_available
  if [ $? -eq 0 ]; then
     # already latest
     return 0
  fi

  is_activity
  if [ $? -ne 0 ]; then
     # activity, so abort
     return 1
  fi

  cd $vardir
  rm -rf security_updates.tar.md5* security_updates.tar.gz* updates 
  wget ftp://ftp.unitrends.com/utilities/security_updates.tar.md5 >>$tmpf 2>&1
  echo "`date` Downloading $tarball " |tee -a $log
  wget ftp://ftp.unitrends.com/utilities/$tarball >>$tmpf 2>&1 
  if [ ! -f $tarball ]; then
     tail -n10 $tmpf |tee -a $log
     echo "Error downloading $tarball" |tee -a $log
     rm -f $tmpf
     return 1
  fi
  rm -f $tmpf
  cat security_updates.tar.md5 >$sum0
  md5sum $tarball >$sum1
  s0=`awk '{ print $1 }' $sum0`
  s1=`awk '{ print $1 }' $sum1`
  rm -f $sum0 $sum1
  if [ $dbg -eq 1 ]; then
     echo "sum0=$s0"
     echo "sum1=$s1"
  fi
  if [ "$s0" != "$s1" ]; then
     echo "Error: Checksums do not match, try again later" |tee -a $log
     return 1
  else
     if [ $dbg -eq 1 ]; then
        echo "checksums match"
     fi
  fi
  echo "`date` Applying $tarball $param" |tee -a $log
  tar -xzf $tarball 
  cd updates
  if [ "$param" = "fg" ]; then
     ./security_updates.sh 
     cd $vardir
     rm -rf security_updates.tar.md5* security_updates.tar.gz* updates 
  else
     # else use background if from Toolbox
     ./security_updates.sh &
  fi
  rv=$?
  cd $mydir
  ps -ef |grep security_updates |grep -v grep 
  cat $vardir/security_version.txt >>$log 
  echo "done"
  return $rv
}

# main logic
export PATH=/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/bin:/usr/bp/bin:/usr/bp/lib/shlib:/root/bin:/usr/local/pgsql/bin
func=$1
parm=$2
ret=0
case $func in
"available")
	checkAvailableUpdate
	ret=$?
	;;
"apply")
	applySecurityUpdate $parm
	ret=$?
	;;
*)
	echo "Usage: $0 <available|apply>"
	;;
esac
exit $ret
